package com.nootropic.ioio.rgbmatrix;

import org.opencv.android.Utils;
import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.core.Point;
import org.opencv.core.Scalar;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;
import org.opencv.highgui.Highgui;
import org.opencv.highgui.VideoCapture;

import android.content.Context;
import android.graphics.Bitmap;
import android.view.SurfaceHolder;

class RGBMatrixView extends ViewBase {
    private Mat cameraMat;
    private Mat subMat = null;
    private Mat outputMat;
    private Scalar black = new Scalar(0, 0, 0, 255);

    public RGBMatrixView(Context context) {
        super(context);
    }

    @Override
    public void surfaceChanged(SurfaceHolder _holder, int format, int width, int height) {
        super.surfaceChanged(_holder, format, width, height);

        synchronized (this) {
            // initialize Mats before usage
            cameraMat = new Mat();
            subMat = new Mat();
            outputMat = new Mat();
        }
    }

    @Override
    protected Bitmap processFrame(VideoCapture capture) {
        int mask = 0;
        if (RGBMatrixActivity.viewMode == RGBMatrixActivity.VIEW_MODE_9BIT) {
            mask = 0x0000001F;
        }
        if (RGBMatrixActivity.viewMode == RGBMatrixActivity.VIEW_MODE_12BIT) {
            mask = 0x0000000F;
        }

        capture.retrieve(cameraMat, Highgui.CV_CAP_ANDROID_COLOR_FRAME_RGBA);
        int w = (int)(cameraMat.size().width);
        int h = (int)(cameraMat.size().height);

        // Set region of interest to 704 x 352 so that the image has the correct aspect ration of 32x16
        // Each pixel in enlarged image is 22x22
        // If your Android phone camera/screen resolution is smaller, choose values that create a 32x16 aspect ratio. 
        int newW = 704;
        int newH = 352;
        subMat = cameraMat.submat((h-newH)/2, h-((h-newH)/2), (w-newW)/2, w-((w-newW)/2));
        
        // create matrix 'outputMat' that is 32x16 pixels
        Imgproc.resize(cameraMat, outputMat, new Size(32, 16));

        // clear the screen on the phone
        Core.rectangle(cameraMat, new Point(0, 0), new Point(cameraMat.cols()-1, cameraMat.rows()-1), black, Core.FILLED);

        // Draw the pixelized image on the phone display so we can see what is going on.
        Point p1, p2;
        for(int y=0;y<16;y++) {
            for(int x=0;x<32;x++) {
                double[] d = outputMat.get(y, x);
                int c_r = ((int)d[0]) & ~mask;
                int c_g = ((int)d[1]) & ~mask;
                int c_b = ((int)d[2]) & ~mask;
                
                Scalar color = new Scalar((double)c_r, (double)c_g, (double)c_b, d[3]);
                int yy = y*22 + 19;
                int xx = x*22;
                p1 = new Point(xx, yy);
                p2 = new Point(xx + 21, yy + 21);
                Core.rectangle(cameraMat, p1, p2, color, Core.FILLED);
            }
        }

        Bitmap bmp = Bitmap.createBitmap(cameraMat.cols(), cameraMat.rows(), Bitmap.Config.ARGB_8888);
        if (Utils.matToBitmap(cameraMat, bmp)) {
            return bmp;
        }
        bmp.recycle();

        return null;
    }

    @Override
    public void run() {
        super.run();

        synchronized (this) {
            // Explicitly deallocate Mats
            if (cameraMat != null)
                cameraMat.release();
            if (outputMat != null)
                outputMat.release();

            cameraMat = null;
            outputMat = null;
        }
    }
    
    public Mat getOutputMat() {
        return outputMat;
    }
}
